#!/usr/bin/env python
#-*- coding: ISO-8859-1 -*-
#pylint: disable-msg=E1101,C0103,R0902,R0903
#Author: Valentin Kuznetsov

"""
Set of commonly used utilities
"""

import sys
import urllib
import urllib2
import types
import time
import traceback

def getArg(kwargs, key, default):
    """provide default value for given key in kwargs dict"""
    arg = default
    if  kwargs.has_key(key):
        try:
            arg = kwargs[key]
            if  type(default) is types.IntType:
                arg = int(arg)
        except NotImplementedError:
            pass
    return arg

def printSize(i):
    """
       Format file size utility, it converts file size into KB, MB, GB, TB, PB units
    """
    try:
        num = long(i)
    except ArithmeticError:
        return "N/A"
    for x in ['', 'KB', 'MB', 'GB', 'TB', 'PB']:
        if  num < 1024.:
            return "%3.1f%s" % (num, x)
        num /= 1024.

def printExcMessage():
    """
       print exception message
    """
    counter = 0
    for m in traceback.format_exc().split("\n"):
        if  m.find(" raise ") != -1:
            counter = 1
            continue
        if  counter:
            print m

def getExcMessage(userMode = 'user'):
    """
       return exception message
    """
    exStr = "%s" % sys.exc_type
    if  userMode == 'dbsExpert':
        return traceback.format_exc()
    if  exStr.find(".") == -1: 
        ex = "raise "
    else:
        ex = exStr.split(".")[-1]
    counter = 0
    msg = ""
    for m in traceback.format_exc().split("\n"):
        if  m.find(ex) != -1:
            counter = 1
            if  ex != "raise ":
                msg += "%s\n" % m
            continue
        if  counter:
            msg += "%s\n" % m
    if  not msg: 
        return traceback.format_exc()
    return msg

def getLFNSize(lfn):
    """return lfn size"""
    url = "https://cmsweb.cern.ch/dbs_discovery/aSearch"
    query = "find file.size where file=%s" % lfn.strip().replace("//","/")
    iParams = {'dbsInst':'cms_dbs_prod_global',
               'html':'0',
               'caseSensitive':'on',
               '_idx':'0',
               'pagerStep':'1',
               'userInput': query,
               'xml':'0',
               'details':'0',
               'cff':'0',
               'method':'dbsapi'}
    data = urllib2.urlopen(url, urllib.urlencode(iParams)).read()
    return data.split()[-1]

def day():
    """return today"""
    return time.strftime("%Y%m%d", time.gmtime(time.time()))

def getPercentageDone(tSize, fSize):
    """format output of percentage"""
    return "%3.1f" % (long(tSize)*100/long(fSize))

class LfnInfoCache(object):
    """
       Simple cache class to keep lfn around
    """
    def __init__(self):
        self.lfnDict = {}
        self.day = day()
        
    def getSize(self, lfn):
        """return lfn size"""
        today = day()
        if  self.day != today:
            self.lfnDict = {}
        if  self.lfnDict.has_key(lfn):
            return self.lfnDict[lfn]
        else:
            lfnsize = getLFNSize(lfn)
            self.lfnDict[lfn] = lfnsize
            return lfnsize
#
# main
#
if __name__ == "__main__":
    size = getLFNSize('/store/data/CRUZET3/Cosmics/RAW/v1/000/050/832/186585EC-024D-DD11-B747-000423D94AA8.root')
    print size
    print getPercentageDone(4*1024*1024, size)
    print printSize(size)
